$def with (page)

<html>
<head>
    <title>ooresults</title>
    <link rel="stylesheet" href="mystatic/style.css">
    <link rel="stylesheet" href="mystatic/style-tab.css">
    <meta http-equiv="Cache-Control" content="no-cache, no-store, must-revalidate" />
    <meta http-equiv="Pragma" content="no-cache" />
    <meta http-equiv="Expires" content="0" />
    <script src="mystatic/w3.js"></script>
</head>
<body onpageshow="myfunction()">

$:page

</body>
</html>
