$def with (message)

<div class="bgy">
    <div style="height: 37vh;" class="vertical-center">
        <p class="s2">$message['controlCard']</p>
    </div>
    <div style="height: 26vh;" class="vertical-center">
        <p class="s1">$message.get('error', ' ')</p>
    </div>
    <div style="height: 37vh;" class="vertical-center">
        <p class="s2">Bitte im WKZ melden</p>
    </div>
</div>
